<?php
if (Manual('Installation', 'install')) {
?>
<p>MemoDisx can work in OS/2 or in Unix systems. If you try to use it under Windows, MemoDisx will
think that it runs under OS/2. It might work but it is neither tested nor supported.</p>

<p>MemoDisx requires the following software tools:</p>
<ul>
<li>Apache 1.3.10 or higher with server-side includes (SSI) enabled</li>
<li>PHP 4 (it will not work with PHP&nbsp;3)</li>
<li>MySQL 3.22.19 or higher</li>
<li>WWW browser supporting frames, CSS2 and JavaScript (JavaScript is recommended but not stricly
required)</li>
</ul>

<p>Other requirements depend on the operating system:</p>
<ul>
<li>OS/2 must have Rexx installed. Apache as well as some other tools require EMX.</li>
<li>Unix version depends on standard utilities <code>ls</code>, <code>df</code>,
<code>mount</code>, <code>umount</code> and <code>isoinfo</code>.</li>
</ul>

<p>If you wish to store the contents of archives you must have <code>InfoZip</code>,
<code>tar</code>, <code>gzip</code> and <code>bzip2</code>. It is not necessary to have all of
them, you will select later which of them you wish to use.</p>

<p>MemoDisx makes full use of <a title="Content negotiation" target="_blank"
href="http://hroch486.icpf.cas.cz/wagner/content-negotiation">Content negotiation</a>. Before you
continue with installation, you must configure Apache for <a title="MultiViews" target="_blank"
href="http://hroch486.icpf.cas.cz/manual/mod/mod_negotiation.html">MultiViews</a>. It is usually
preconfigured but the distributed configuration file may contain a bug in definition of the Czech
language. The correct ISO code is <b>cs</b>, not <b>cz</b>!</p>

<p>MemoDisx is now available in English and Czech. Be sure that your configuration file includes
<code>negotiation_module</code> and contains the following directives:</p>

<pre>
AddLanguage en .en
AddLanguage cs .cs
AddCharset iso-8859-2 .iso8859-2
</pre>

<p>It is allowed to assign more extensions to each language. You can also define other languages.</p>

<p>You should also configure languages in your WWW browser. The primary MemoDisx language is
English. Be sure that your WWW browser contains generic English.</p>

<p>MemoDisx supports only one database for all users. If you must use several databases, you must
install several distinct copies of MemoDisx.</p>

<p>First the database table must be created by someone with administrative right for MySQL. The
commands are prepared in the <code>incl.dbcreate.txt</code> file. Be sure that you first edit at
least the password.</p>

<p>Now you can prepare files for Apache. Copy the whole directory to the location where Apache
expects PHP files.
<b>This directory must have write permission for the user under which Apache runs!</b>
Do not forget to set <code>umask&nbsp;0</code>, otherwise you will not be able to grant write
permission to all users.
Be sure that you have server-side includes enabled. Afterwards you must
configure MemoDisx using the statements from <code>incl.htaccess</code> (put them to the main
configration file). The contents is explained in the following text.</p>

<pre>
# MemoDisx configuration
# ======================
# PHP4 and SSI must be enabled.
# MySQL must be installed on the same host.
# AddLanguage and AddCharset directives must be properly set.
# The system will NOT work without register_globals off.
# SSL is not necessary but recommended unless you access MemoDisx
# from your local computer and you are not sure that there is
# no TCP/IP spy which could reveal your password to someone else.

&lt;Directory @@PHPDocumentRoot@@/memodisx&gt;
</pre>

<p>Replace the directory name with the one used on your server.</p>

<pre>
  AllowOverride None
  Options +MultiViews +IncludesNOEXEC
  DirectoryIndex index
</pre>

<p>These directives configure SSI and MultiViews for this directory.</p>

<pre>
  &lt;IfModule mod_php4.c&gt;
    # AddType application/x-httpd-php php
    php_flag magic_quotes_gpc off
    php_flag magic_quotes_runtime off
    php_admin_flag register_globals off
  &lt;/IfModule&gt;
</pre>

<p>The great advantage of PHP is that its parameters can be set in the Apache
configuration file on the per-directory basis. We will thus not place the
settings to <code>php.ini</code> in order not to disturb other scripts.</p>

<p>MemoDisx does not register globals for security reasons and handles quotes itself. The
<code>AddType</code> directive usually appears in the main config and need not be repeated here.</p>

<pre>
  &lt;IfModule mod_ssl.c&gt;
    SSLRequireSSL
  &lt;/IfModule&gt;
</pre>

<p>SSL is not required but is recommended unless you are sure that no network spy can read your
password.</p>

<pre>
  &lt;IfModule mod_expires.c&gt;
    ExpiresActive On
    ExpiresDefault M3
    ExpiresByType text/css "access 1 day"
    ExpiresByType image/gif "access 1 month"
    ExpiresByType image/jpeg "access 1 month"
  &lt;/IfModule&gt;
</pre>

<p>The pages are dynamically generated, therefore their expiration must be properly set. Without
this module MemoDisx may not load new pages but display the old contents.</p>

<pre>
  &lt;IfModule mod_headers.c&gt;
    &lt;FilesMatch "\.php$"&gt;
      Header append Pragma "no-cache"
      Header set Cache-Control "no-cache, must-revalidate, private"
    &lt;/FilesMatch&gt;
  &lt;/IfModule&gt;
</pre>

<p>This part prevents caching the contents in the WWW browser.</p>

<pre>
  &lt;FilesMatch "incl\."&gt;
    Order allow,deny
    Deny from all
  &lt;/FilesMatch&gt;
</pre>

<p>These files contain functions and sensitive information which must not be displayed.</p>

<pre>
  AuthGroupFile @@SecurityFiles@@/groups
  AuthUserFile @@SecurityFiles@@/users
  AuthName "MemoDisx"
  AuthType Basic
  require valid-user
  # require group memodisx
  # require user memodisx Alice Bill Dinah Jabberwock Tweedledum Tweedledee
</pre>

<p>In order to prevent unautorized use it is recommended (but not strictly required) to adopt
access restrictions. You can allow access to all users, to a group of users or to specified
users. Superuser <b>memodisx</b> has special rights which are described in other parts of the
manual.</p>

<pre>
&lt;/Directory&gt;
</pre>

<p>You can limit the right to read the CD by creating file <code>incl.readcd.config.php</code>.
Sample of such a file is distributed as <code>incl.readcd-x.config.php</code>. You can use the
following predefined constants:</p>

<table>
<tr>
<th align="left">_superuser_</th>
<td>Superuser name (memodisx)</td>
</tr>
<tr>
<th align="left">_x_user_</th>
<td>Current user</td>
</tr>
<tr>
<th align="left">_x_user_ip_</th>
<td>IP address of the user's computer</td>
</tr>
<tr>
<th align="left">_server_ip_</th>
<td>IP address of the server</td>
</tr>
<tr>
<th align="left">_notallowed_</th>
<td>Error message</td>
</tr>
</table>

<p>If you wish to make a demo version, save a copy of <code>incl.demo-x.php</code> as
<code>incl.demo.php</code>.</p>

<p>If you have not done so, restart Apache, save a copy of <code>incl.install.php</code> as
<code>install.php</code> and run the <a title="Installation" target="_top"
href="install">installation script</a>. It will ask you a few questions. You must specify the host
name, database name, database user and database password for MySQL. Afterwards you specify which
archiving and compression programs are available on your system. GNU tar differs from other
versions by option <code>-z</code> which allows reading <code>.tar.gz</code> files directly. This
is not supported in OS/2. Compression utilities are used only if tar is specified. Unix systems
usually contain symbolic links with names <code>gunzip</code> and <code>bunzip2</code>. If you do
not have them, you must select utilities with the <code>-d</code> option. Finally you select
all drives from which you could read a CD (drive letters in OS/2, mount points in Unix - do
not use device names).</p>

<p><b>The MemoDisx directory must have write permission for the user under which Apache runs!</b></p>

<p>Installation will create file <code>incl.constants.php</code> in your MemoDisx directory. Do not
edit the file! Only if you are not satisfied with the order of CD drives, change the
<code>$_cd_</code> array.</p>

<p>It is recommended that you first play a bit with the program so that you learn how to use it.
When you feel that you understand the program, save a copy of <code>incl.delete.php</code> as
<code>delete.php</code> and use it to <a title="Delete the database" target="_top"
href="delete">delete the whole contents of the database</a>. Finally delete
<code>install.php</code> and <code>delete.php</code> in order to prevent their use by mistake.</p>

<?php
}
?>
