<?php
require 'incl.memodisx.php';

function DisplayCDcontents($cdsource) {
echo "<pre>";
if (_linux_mode_) {
  $dev = MountCD($cdsource);
  passthru("isoinfo -d -i $dev | grep '^Volume id:'");
  exec("ls -la $cdsource/*", $ls);
}
else exec("dir $cdsource /s", $ls);
while (list($k, $v) = each($ls)) echo htmlspecialchars($v), "\n";
echo "</pre>\n\n";
}

# MountCD in Linux
function MountCD($cdsource) {
exec("df | grep '$cdsource$'", $x);
if (count($x) <= 0) {
  exec("mount $cdsource");
  exec("df | grep '$cdsource$'", $x);
}
if (count($x) <= 0) die("CD error");
$d = explode(' ', $x[0]);
return $d[0];
}

# Get volume name
function VolumeCD($cdsource) {
$vol = '';
if (_linux_mode_) {
  $dev = MountCD($cdsource);
  exec("isoinfo -d -i $dev | grep '^Volume id:'", $x);
  if (preg_match('/^Volume id:\s*(.*)$/', $x[0], $z)) $vol = $z[1];
}
else {
  exec("dir $cdsource", $x);
  while (!$vol && (list($k, $v) = each($x))) {
    $v = trim($v);
    if ($v && preg_match('/^The volume label in drive [A-Z] is (.*)\.$/', $v, $z)) $vol = $z[1];
  }
}
return $vol;
}

# Eject CD
function EjectCD($cdsource) {
if (_linux_mode_) exec("umount $cdsource");
else exec("eject $cdsource");
}

# Class for archives
class Archive {
  var $disk;      # disk node
  var $mode;      # archive mode
  var $state;     # state
  var $dirs;      # directories
  var $parent;    # parent node

  function Archive($disk, $parent, $mode) {
    $this->disk = $disk;
    $this->parent = $parent;
    $this->mode = $mode;
    $this->state = $this->mode != 'zip';
  }

  function SplitLine($line) {
    $x = preg_split('/\s+/', $line, 99, PREG_SPLIT_NO_EMPTY);
    switch ($this->mode) {
    case 'zip':
      if (preg_match('/^[- ]+$/', $line)) $this->state = !$this->state;
      elseif ($this->state) {
        $a['size'] = $x[0];
        $a['datetime'] = $x[4] . ' ' . $x[5];
        if (preg_match('/\/$/', $x[7])) {
          $a['type'] = 'd';  $a['dir'] = substr($x[7], 0, -1);
        }
        else {
          $a['type'] = 'f';
          $a['name'] = basename($x[7]);
          $a['dir'] = dirname($x[7]);
        }
      }
      break;
    case 'tar':
      if (preg_match('/^[-dl]([-r][-w][-x]){3}$/', $x[0])) {
        $n = count($x) - 1;
        $a['type'] = substr($x[0], 0, 1);
        if ($a['type'] == 'l') $n -= 2;
        $a['size'] = $x[2];
        $a['datetime'] = $n == 5 ? $x[3] . ' ' . $x[4] : $this->os2TarDate($x);
        $a['type'] = substr($x[0], 0, 1);
        if ($a['type'] != 'd' && $a['type'] != 'l') $a['type'] = 'f';
        if ($a['type'] == 'd') $a['dir'] = substr($x[$n], 0, -1);
        else {
          $a['name'] = basename($x[$n]);
          $a['dir'] = dirname($x[$n]);
        }
      }
      break;
    }
    return $a;
  }

  function os2TarDate($x) {
    $months = array(
      'jan' => 1, 'feb' => 2, 'mar' => 3, 'apr' => 4, 'may' => 5, 'jun' => 6,
      'jul' => 7, 'aug' => 8, 'sep' => 9, 'oct' => 10, 'nov' => 11, 'dec' => 12);
    return $x[6] . '-' . $months[strtolower($x[3])] . '-' . $x[4] . ' ' . $x[5];
  }

  function GetNode($d) {
    $node = 0;
    $dirinfo = $this->dirs[$d];
    if ($dirinfo && $dirinfo['node']) $node = $dirinfo['node'];
    if (!$node) {
      if (dirname($d) != '.') $parent = $this->GetNode(dirname($d));
      else $parent = $this->parent;
      if ($dirinfo && $dirinfo['datetime']) $mtime = '"' . $dirinfo['datetime'] . '"';
      else $mtime = 0;
      $node = AddNode(basename($d), 'ArchDir', $this->disk, $parent, $mtime);
      $this->dirs[$d]['node'] = $node;
    }
    return $node;
  }

  function AddLine($line) {
    $a = $this->SplitLine($line);
    if (!$a) return;
    if ($a['type'] == 'd') {
      if (!$this->dirs[$a['dir']]) $this->dirs[$a['dir']]['datetime'] = $a['datetime'];
    }
    else {
      $parent = $a['dir'] != '.' ? $this->GetNode($a['dir']) : $this->parent;
      $ntype = $a['type'] == 'f' ? 'ArchFile' : 'Link';
      $mtime = '"' . $a['datetime'] . '"';
      AddNode($a['name'], $ntype, $this->disk, $parent, $mtime, $a['size']);
    }
  }
}

# Add directory contents
function AddContents($path, $disk, $parent, $archmasks) {
echo "<p>Dir <code>", htmlspecialchars($path), "</code></p>\n";  flush(stdout);
$wdir = substr($path, -1) == ':' ? "$path/" : $path;
$dh = opendir($wdir);
clearstatcache();
unset($dirs);
while ($nm = readdir($dh)) {
  if ($nm == '.' || $nm == '..') continue;
  $fn = "$path/$nm";
  if (!is_link($fn)) {
    $mtime = strftime('%Y%m%d%H%M%S', filemtime($fn));
    if (is_dir($fn)) $dirs[] = $nm;
    elseif (is_file($fn)) {
      $ntype = 'File';
      # Test for archives
      if (isset($archmasks) && is_array($archmasks)) {
        reset($archmasks);
        while ($ntype == 'File' && (list($k, $am) = each($archmasks))) {
          $mask = $am['mask'];
          if (preg_match("/$mask/", $nm)) {
            echo "<p>Arch <code>", htmlspecialchars($fn), "</code></p>\n";  flush(stdout);
            $cmd = sprintf($am['command'], $fn);
            $ntype = 'Archive';  $amode = $am['mode'];
            unset($lines);       exec($cmd, $lines);
          }
        }
      }
      $node = AddNode($nm, $ntype, $disk, $parent, $mtime, filesize($fn));
      if ($ntype == 'Archive') {
        # Process an archive
        $archive = new Archive($disk, $node, $amode);
        reset($lines);
        while (list($k, $v) = each($lines)) $archive->AddLine($v);
        unset($archive);
      }
    }
    elseif (is_link($fn)) {
      AddNode($nm, 'Link', $disk, $parent, $mtime, filesize($fn));
    }
  }
}
closedir($dh);
if (!isset($dirs)) return;
reset($dirs);
while (list($k, $d) = each($dirs)) {
  $node = AddNode($d, 'Directory', $disk, $parent, $mtime);
  AddContents("$path/$d", $disk, $node, $archmasks);
}}

title();
if (file_exists('incl.readcd.config.php')) {
  include('incl.readcd.config.php');
}
if (file_exists('incl.demo.php')) {
  include('incl.demo.php');
}
elseif ($_SERVER['REQUEST_METHOD'] == 'GET') {
  if (!$_GET['cdsource']) {
?>
<form method="GET" action="readcd"><center>
<p><?php echo _select_cd_; ?>:
<select name="cdsource" size=1>
<?php
    reset($_cd_);
    while (list($k, $v) = each($_cd_)) echo "<option>$v</option>\n";
?>
</select>
<input type="submit" />
</p></center></form>
<?php
  }
  else {
    echo _reading_; flush(stdout);
    DisplayCDcontents($_GET['cdsource']);
?>
<form method="POST" action="readcd">
<p><input type=checkbox name="eject" />
<?php echo _eject_; ?></p>
<p><?php echo _comment_cd_; ?>:
<input name="comment" size=64 maxlength=255 /></p>
<p><input type="hidden" size=0 name="cdsource" value="<?php echo $_GET['cdsource']; ?>" />
<input type=submit /></p></form>
<?php
  }
}
elseif ($_SERVER['REQUEST_METHOD'] == 'POST') {
  echo _reading_;
  $vol = VolumeCD($_POST['cdsource']);
  if (!$vol) die('<p class="error">' . _no_cd_ . $_POST['cdsource']);
  if ($vol == _cd_volume_) {
    md_mysql('insert nonames SET noname=NULL;');
    $vol .= '_' . mysql_insert_id(_sq_);
  }
  echo "<p>Vol <code>", htmlspecialchars($vol), "</code></p>\n";
  flush(stdout);

  # Prepare masks, the first match will stop reading masks
  if (_zip_) {
    $ArchMasks[] = array(
      'mask' => '\.[zZ][iI][pP]$',
      'command' => 'unzip -v %s',
      'mode' => 'zip');
  }
  if (_tar_) {
    $ArchMasks[] = array(
      'mask' => '\.[tT][aA][rR]$',
      'command' => 'tar -tvf %s',
      'mode' => 'tar');
    if (_tar_ > 0) {
      $ArchMasks[] = array(
        'mask' => '\.[tT]([aA][rR]\.)?[gG][zZ]$',
        'command' => 'tar -tvzf %s',
        'mode' => 'tar');
    }
    if (_bzip2_) {
      $unz = _bzip2_ > 0 ? 'bunzip2 -ck' : 'bzip2 -cdk';
      $ArchMasks[] = array(
        'mask' => '\.[tT][aA][rR]\.[bB][zZ]2$',
        'command' => "$unz %s | tar -tvf -",
        'mode' => 'tar');
    }
    if (_gzip_) {
      $unz = _gzipg_ > 0 ? 'gunzip -c' : 'gzip -cd';
      $ArchMasks[] = array(
        'mask' => '\.[tT]([aA][rR]\.)?[gG][zZ]$',
        'command' => "$unz %s | tar -tvf -",
        'mode' => 'tar');
    }
  }
  LockTables('write');
  $disk = AddNode($vol, 'Volume');
  $owner = $_SERVER['REMOTE_USER'];
  if ($owner) AddNode($owner, 'Owner', $disk, $disk, 0);
  if (trim($_POST['comment'])) AddNode(trim($_POST['comment']), 'Comment', $disk, $disk, 0);
  AddContents($_POST['cdsource'], $disk, $disk, $ArchMasks);
  UnlockTables();
  echo "<hr />\n<center>\n";
  DisplaySummary($disk);
  if ($_POST['eject'] == 'on') EjectCD($_POST['cdsource']);
  echo '<p>', _finished_, "</p>\n</center>\n";
}
?>
</body>
</html>
